
'use strict'

/**
 * Encodes a URI according to a set of template variables. Variables will be
 * passed through encodeURIComponent.
 * @param {string} pathTemplate The path with template variables e.g. '/foo/$bar'.
 * @param {Object} variables The key/value pairs to replace the template
 * variables with. E.g. { "$bar": "baz" }.
 * @return {string} The result of replacing all template variables e.g. '/foo/baz'.
 */
export const encodeUri = function(pathTemplate, variables) {
  for (const key in variables) {
    if (!variables.hasOwnProperty(key)) {
      continue
    }
    pathTemplate = pathTemplate.replace(
      key, encodeURIComponent(variables[key]),
    )
  }
  return pathTemplate
}

function addHTMLSuffix(url) {
  if (typeof url !== 'string') throw new Error('wx.redirectTo: invalid url:' + url)
  const urlArr = url.split('?')
  urlArr[0] += '.html'
  return typeof urlArr[1] !== 'undefined' ? urlArr[0] + '?' + urlArr[1] : urlArr[0]
}

export function getRealRoute(pathname, url, isHtml = true) {
  if (!pathname) return url
  isHtml && (url = isHtml ? addHTMLSuffix(url) : url)
  if (url.indexOf('/') === 0) return url.substr(1)
  if (url.indexOf('./') === 0) return getRealRoute(pathname, url.substr(2), !1)
  let index
  let urlArrLength
  const urlArr = url.split('/')
  for (index = 0, urlArrLength = urlArr.length; index < urlArrLength && urlArr[index] === '..'; index++);
  urlArr.splice(0, index)
  const pathArr = pathname.length > 0 ? pathname.split('/') : []
  pathArr.splice(pathArr.length - index - 1, index + 1)
  const newPathArr = pathArr.concat(urlArr)
  return newPathArr.join('/')
}

export const encodeUrlQuery = function(url) { // 把url中的参数encode
  if (typeof url === 'string') {
    const urlArr = url.split('?')
    const urlPath = urlArr[0]
    const queryParams = (urlArr[1] || '').split('&').reduce(function(res, cur) {
      if (typeof cur === 'string' && cur.length > 0) {
        const curArr = cur.split('=')
        const key = curArr[0]
        const value = curArr[1]
        res[key] = value
      }
      return res
    }, {})
    const urlQueryArr = []
    for (const i in queryParams) {
      Object.prototype.hasOwnProperty.call(queryParams, i) && urlQueryArr.push(i + '=' + encodeURIComponent(queryParams[i]))
    }
    return urlQueryArr.length > 0 ? urlPath + '?' + urlQueryArr.join('&') : url
  }
  return url
}

export const getDataType = function(data) { // get data type
  return Object.prototype.toString.call(data).split(' ')[1].split(']')[0]
}

export const convertObjectValueToString = function convertObjectValueToString(obj) { // 把对象元素都转成字符串
  return Object.keys(obj).reduce(function(res, cur) {
    typeof obj[cur] === 'string'
      ? res[cur] = obj[cur]
      : typeof obj[cur] === 'number'
        ? res[cur] = obj[cur] + ''
        : res[cur] = Object.prototype.toString.apply(obj[cur])
    return res
  }, {})
}

export function urlEncodeFormData(data, needEncode) { // 把对象生成queryString
  if (typeof (data) !== 'object') return data
  const tmpArr = []
  for (const o in data) {
    if (Object.prototype.hasOwnProperty.call(data, o)) {
      if (needEncode) {
        try {
          tmpArr.push(encodeURIComponent(o) + '=' + encodeURIComponent(data[o]))
        } catch (t) {
          tmpArr.push(o + '=' + data[o])
        }
      } else tmpArr.push(o + '=' + data[o])
    }
  }
  return tmpArr.join('&')
}

export const addQueryStringToUrl = function(originalUrl, newParams, needEncode) { // 生成url t:param obj
  if (typeof originalUrl === 'string' && typeof newParams === 'object' && newParams) {
    const urlComponents = originalUrl.split('?')
    const host = urlComponents[0]
    const oldParams = (urlComponents[1] || '').split('&').reduce(function(res, cur) {
      if (typeof cur === 'string' && cur.length > 0) {
        const curArr = cur.split('=')
        const key = curArr[0]
        const value = curArr[1]
        res[key] = value
      }
      return res
    }, {})
    const refinedNewParams = Object.keys(newParams).reduce(function(res, cur) {
      typeof (newParams[cur]) === 'object'
        ? res[encodeURIComponent(cur)] = encodeURIComponent(JSON.stringify(newParams[cur]))
        : res[encodeURIComponent(cur)] = encodeURIComponent(newParams[cur])
      return res
    }, {})
    const formData = urlEncodeFormData(Object.assign(oldParams, refinedNewParams), needEncode)
    if (formData) {
      return host + '?' + formData
    }
    return host
  }
  return originalUrl
}

export const base64ToArrayBuffer = function base64ToArrayBuffer(str) {
  const atobStr = atob(str)
  const leng = atobStr.length
  const arr = new Uint8Array(leng)
  for (let r = 0; r < leng; r++) arr[r] = atobStr.charCodeAt(r)
  return arr.buffer
}

const chars = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/='

export const atob = window.atob ||
  function(input) {
    const str = (String(input)).replace(/[=]+$/, '') // #31: ExtendScript bad parse of /=
    if (str.length % 4 === 1) {
      throw new InvalidCharacterError('\'atob\' failed: The string to be decoded is not correctly encoded.')
    }

    let bc = 0; let bs; let buffer; let idx = 0; let output = ''
    for (;buffer = str.charAt(idx++); // eslint-disable-line no-cond-assign
      // character found in table? initialize bit storage and add its ascii value;
      ~buffer && (bs = bc % 4 ? bs * 64 + buffer : buffer,
      // and if not first of each 4 characters,
      // convert the first 8 bits to one ascii character
      bc++ % 4) ? output += String.fromCharCode(255 & bs >> (-2 * bc & 6)) : 0
    ) {
      // try to find character in table (0-63, not found => -1)
      buffer = chars.indexOf(buffer)
    }
    return output
  }
